// src/normalize.ts
import { existsSync, readdirSync } from "node:fs";
import path from "node:path";

// src/utils.ts
var isNonEmptyArray = (arr) => Array.isArray(arr) && arr.length > 0;
var containsSingleAsterisk = (str) => (str.match(/\*/g) || []).length === 1;

// src/normalize.ts
var defaultConditions = ["node-addons", "node", "import", "require", "default"];
var checkExportsConfiguration = (exports) => {
  if (typeof exports === "object") {
    if (Object.keys(exports).length === 0) {
      throw new Error("No entries found in exports map");
    }
    if (!Array.isArray(exports)) {
      const exportsKeys = Object.keys(exports);
      const keysStartingWithPeriod = exportsKeys.filter((key) => key.startsWith("."));
      if (keysStartingWithPeriod.length !== 0 && keysStartingWithPeriod.length !== exportsKeys.length) {
        throw new Error("Invalid exports configuration");
      }
    }
  }
};
var matchConditions = (exports, conditions) => {
  if (Array.isArray(exports)) {
    if (exports.length === 0) {
      return null;
    }
    let lastError;
    for (const subEntry of exports) {
      try {
        const value = matchConditions(subEntry, conditions);
        if (typeof value !== "undefined") {
          return value;
        }
      } catch (error) {
        lastError = error;
      }
    }
    if (typeof lastError === "object") {
      throw lastError;
    }
  } else if (exports && typeof exports === "object") {
    for (const [key, subEntry] of Object.entries(exports)) {
      if (conditions.has(key)) {
        const value = matchConditions(subEntry, conditions);
        if (typeof value !== "undefined") {
          return value;
        }
      }
    }
  } else if (typeof exports === "string" && !exports.startsWith("./")) {
    throw new Error(`Invalid target '${exports}'`);
  } else {
    return exports;
  }
};
var expandWildcardEntry = (wildcardKey, wildcardValue, cwd) => {
  const [wildcardValueBase, wildcardValueTrail] = wildcardValue.split("*");
  const directoryToExpand = path.resolve(cwd, wildcardValueBase);
  if (!existsSync(directoryToExpand)) {
    throw new Error(`Directory '${directoryToExpand}' not found`);
  }
  const expandedEntries = {};
  const directoryEntries = readdirSync(directoryToExpand, { withFileTypes: true, recursive: true });
  for (const directoryEntry of directoryEntries) {
    if (directoryEntry.isFile()) {
      const expandedPath = path.join(directoryEntry.path, directoryEntry.name);
      const relativeExpandedPath = path.relative(directoryToExpand, expandedPath);
      if (relativeExpandedPath.endsWith(wildcardValueTrail)) {
        const patternMatch = relativeExpandedPath.slice(0, relativeExpandedPath.indexOf(wildcardValueTrail));
        expandedEntries[wildcardKey.replace("*", patternMatch)] = wildcardValue.replace("*", patternMatch);
      }
    }
  }
  return expandedEntries;
};
var expandWildcardEntries = (flattenedEntries, cwd) => {
  let expandedEntries = {};
  const nonWildcardEntries = {};
  const excludedWildcardKeys = [];
  for (const [key, value] of Object.entries(flattenedEntries)) {
    if (key.includes("*") && cwd) {
      if (!containsSingleAsterisk(key)) {
        throw new Error(`Invalid subpath pattern '${key}'`);
      }
      if (value === null) {
        excludedWildcardKeys.push(key);
      } else if (!containsSingleAsterisk(value)) {
        throw new Error(`Multiple '*' in '${value}' are not supported`);
      } else {
        expandedEntries = {
          ...expandedEntries,
          ...expandWildcardEntry(key, value, cwd)
        };
      }
    } else if (value !== null) {
      nonWildcardEntries[key] = value;
    }
  }
  for (const excludedKey of excludedWildcardKeys) {
    const [excludedKeyBase, excludedKeyTrail] = excludedKey.split("*");
    for (const expandedKey of Object.keys(expandedEntries)) {
      if (expandedKey.startsWith(excludedKeyBase) && expandedKey.endsWith(excludedKeyTrail) && excludedKeyBase.length + excludedKeyTrail.length <= expandedKey.length) {
        delete expandedEntries[expandedKey];
      }
    }
  }
  return {
    ...nonWildcardEntries,
    ...expandedEntries
  };
};
var getFlattenedEntries = (exports, conditions, cwd) => {
  let flattenedEntries = {};
  const singleSubpath = matchConditions(exports, conditions);
  if (typeof singleSubpath === "string") {
    flattenedEntries["."] = singleSubpath;
  } else {
    for (const [key, subEntry] of Object.entries(exports)) {
      if (!key.startsWith(".")) {
        continue;
      }
      const value = matchConditions(subEntry, conditions);
      if (value !== void 0) {
        flattenedEntries[key] = value;
      }
    }
  }
  return expandWildcardEntries(flattenedEntries, cwd);
};
var normalizeExports = (exports, options) => {
  if (!exports) {
    throw new Error("Exports is not defined");
  }
  checkExportsConfiguration(exports);
  const { conditions, cwd } = options || {};
  const normalizedExports = getFlattenedEntries(
    exports,
    new Set(isNonEmptyArray(conditions) ? conditions : defaultConditions),
    cwd
  );
  return normalizedExports;
};
export {
  normalizeExports
};
